if (typeof Alloy === 'undefined') {
    var Alloy = require('alloy');
}
if (typeof Backbone === 'undefined') {
    var Backbone = require('alloy/backbone');
}
if (typeof _ === 'undefined') {
    var _ = require('alloy/underscore')._;
}

var Q = require('q');
var net = require('net');
var cache = require('cache');

function empty(v) {
    return !v || typeof v == "string" && v == "" || v == null;
}

var DataSync = function () {

    var Usuarios = null;
    var Orcamentos = null;
    var OrcamentoDetalhes = null;

    var UsuariosDB = {};
    var DetalhesDB = {};

    return {
        sync: function (model, success, error, progress) {
            var model = model || "Usuario",
                success = success || function () {},
                error = error || function () {},
                progress = progress || function () {};

            switch (model) {
                case "Usuario":
                    return this.syncUsuario(progress).then(success, error);
                    break;
                case "Orcamento":
                    return this.syncOrcamento(progress).then(success, error);
                    break;
            }
        },
        fetchDataIfNeeded: function (_options) {
            var self = this;
            var deferred = Q.defer();
            var online = Alloy.Globals.isOnline();

            var md5sum = null,
                modified = null;

            var _options = _.extend({}, _options || {});

            var currentFetch = cache.getItem(_options.url_fetch);
            if (currentFetch.cache && currentFetch.md5sum != null && currentFetch.md5sum != "") {
                md5sum = currentFetch.md5sum;
                modified = currentFetch.created;
            }

            net.fetchData({
                url: _options.url_fetch,
                success: function (data) {
                    if (data.success && "responseJSON" in data) {
                        try {
                            if ("md5sum" in data.responseJSON) {
                                return deferred.resolve({
                                    responseData: data.responseJSON.data,
                                    actual_md5sum: data.responseJSON.md5sum,
                                    isFresh: data.responseJSON.md5sum != md5sum
                                });
                            }
                        } catch (ex) {}
                    }

                    return deferred.resolve({
                        responseData: false,
                        actual_md5sum: md5sum,
                        isFresh: false
                    });
                },
                error: function (data) {
                    return deferred.reject({
                        responseData: false,
                        actual_md5sum: md5sum,
                        isFresh: false
                    });
                }
            });

            return deferred.promise;
        },
        save: function (model, data) {
            var newEntry = Alloy.createModel(model, data);
            newEntry.save();

            return newEntry.id;
        },
        saveAll: function (rows, translateLine, deleteOthers, progress) {
            deleteOthers = typeof deleteOthers == "undefined" ? true : !!deleteOthers;

            var self = this;
            var rows = rows || [];
            var progress = progress || function () {};
            var translateLine = translateLine || function (row) {
                return row;
            };

            var updatedIds = {};

            var db = Alloy.Globals.openDatabase();
            function internalSave(model, data) {
                var newEntry = Alloy.createModel(model, data);

                var table = newEntry.config.adapter.collection_name,
                    sql,
                    columns = newEntry.config.columns;

                var names = [],
                    values = [],
                    q = [],
                    u = [],
                    qv = [];
                for (var k in columns) {
                    if (k in data) {
                        names.push(k);
                        values.push(data[k]);
                        q.push("?");
                        qv.push(!isNaN(data[k]) ? data[k] : "'" + data[k] + "'");
                        u.push(k + " = ?");
                    }
                }

                sql = "REPLACE INTO " + table + " (" + names.join(",") + ") VALUES (" + q.join(",") + ");";
                db.execute(sql, values);
                if (!("id" in data) || null === data['id']) {
                    var sqlId = "SELECT last_insert_rowid();";
                    var rs = db.execute(sqlId);
                    if (rs && rs.isValidRow()) {
                        data['id'] = rs.field(0);
                    } else Ti.API.warn("Unable to get ID from database for model: " + model);
                    rs && rs.close();
                }

                return "id" in data ? data['id'] : false;
            };

            Ti.API.log("[DATA SYNC] SAVING NEW ROWS");

            db.execute("BEGIN IMMEDIATE TRANSACTION");

            var db_rows = _.isArray(rows) ? rows : [rows];
            _.each(db_rows, function (row, index) {
                var newRow = translateLine(row);
                if (newRow) {
                    _.each(newRow, function (data, model) {
                        if (!updatedIds[model]) updatedIds[model] = [];

                        if (_.isArray(data)) {
                            _.each(data, function (subdata, subindex) {
                                var newEntry = internalSave(model, subdata);
                                if (newEntry) updatedIds[model].push(!isNaN(subdata.id) ? subdata.id : "'" + subdata.id + "'");
                            });
                        } else {
                            var newEntry = internalSave(model, data);
                            if (newEntry) updatedIds[model].push(!isNaN(data.id) ? data.id : "'" + data.id + "'");
                        }
                    });
                }
                progress(index + 1, db_rows.length);
            });

            if (deleteOthers) {
                Ti.API.warn("[DATA SYNC] DELETE POSSIBLE OLD ROWS");

                _.each(updatedIds, function (ids, model) {
                    var theModel = Alloy.Collections.instance(model);
                    if (theModel) {
                        var table = theModel.config.adapter.collection_name;
                        var delete_sql = "DELETE FROM " + table;
                        if (ids.length) {
                            delete_sql += " WHERE id NOT IN(" + ids.join(", ") + ")";
                        }

                        db.execute(delete_sql);
                    }
                });
            }

            db.execute("COMMIT TRANSACTION");
            db.close();

            return updatedIds;
        },
        serializeUsuario: function (usuarioModel) {
            if (!Usuarios) Usuarios = Alloy.Collections.instance('Usuario');

            var usuario = {};
            usuario["Usuario"] = usuarioModel.toJSON();

            return usuario;
        },
        syncUsuario: function (progress) {
            var self = this;
            var deferred = Q.defer();
            var online = Alloy.Globals.isOnline();

            if (!Usuarios) Usuarios = Alloy.Collections.instance('Usuario');

            function parseFetch() {
                var results = [];
                Usuarios.map(function (usuarioModel) {
                    results.push(self.serializeUsuario(usuarioModel));
                });

                return deferred.resolve(results);
            }

            Usuarios && Usuarios.fetch({
                success: function (collection) {
                    if (!online && collection.length > 0) {
                        Ti.API.log("Cache offline: usuarios");
                        return parseFetch();
                    }

                    self.fetchDataIfNeeded({
                        url_fetch: Alloy.Globals.getApiUri("/usuarios/api/list.json")
                    }).then(function (response) {
                        if (response.isFresh && response.responseData) {
                            self.saveAll(response.responseData, function translateLine(row) {
                                row['Usuario']['password'] = "";
                                return row;
                            }, true, progress);

                            Usuarios.fetch();
                            return parseFetch();
                        } else {
                            Ti.API.log("Cache checked online: usuarios");
                            return parseFetch();
                        }
                    });
                }
            });

            return deferred.promise;
        },
        usuarioEquipe: function (usuario_id) {
            var equipe = [usuario_id || 0];

            Alloy.Collections.instance('Usuario').fetch({
                query: "SELECT id FROM usuarios WHERE gerente_id = " + equipe[0],
                success: function (collection) {
                    if (collection.length) {
                        collection.map(function (usuarioModel) {
                            equipe.push(usuarioModel.get("id") || 0);
                        });
                    }
                }
            });

            return equipe;
        },
        fetchOrcamentoDetalhe: function (ids) {
            if (!OrcamentoDetalhes) OrcamentoDetalhes = Alloy.Collections.instance('OrcamentoDetalhe');

            var detalhe_query = "SELECT d.id, d.created, d.modified, d.orcamento_id, d.data, d.subtotal, d.total, d.desconto, d.req_desconto, d.req_desconto_aprovador_id, d.req_desconto_data_aprovado FROM " + OrcamentoDetalhes.config.adapter.collection_name + " AS d ";

            detalhe_query += "JOIN(" + "SELECT orcamento_id, MAX(data)AS max_data FROM orcamento_detalhes GROUP BY orcamento_id ORDER BY max_data DESC" + ") AS od ON d.data = od.max_data AND d.orcamento_id = od.orcamento_id ";

            if (ids.length == 1) {
                detalhe_query += "WHERE d.orcamento_id = " + ids[0];
            } else {
                detalhe_query += "WHERE d.orcamento_id IN(" + ids.join(",") + ")";
            }

            DetalhesDB = {};

            OrcamentoDetalhes.fetch({
                query: detalhe_query,
                success: function (collection) {
                    if (collection.length) {
                        collection.map(function (detalheModel) {
                            var detalhe = detalheModel.toJSON();
                            DetalhesDB[detalhe.orcamento_id] = detalhe;
                        });
                    }
                }
            });
        },
        serializeOrcamento: function (orcamentoModel) {
            if (!Usuarios) Usuarios = Alloy.Collections.instance('Usuario');
            if (!Orcamentos) Orcamentos = Alloy.Collections.instance('Orcamento');
            if (!OrcamentoDetalhes) OrcamentoDetalhes = Alloy.Collections.instance('OrcamentoDetalhe');

            var orcamento = {};
            orcamento["Orcamento"] = orcamentoModel.toJSON();
            orcamento["OrcamentoDetalhe"] = [];
            orcamento["Usuario"] = {};

            var orcamento_id = orcamento.Orcamento.id || 0;

            if (!empty(orcamento.Orcamento.seminovo)) {
                try {
                    orcamento.Orcamento.seminovo = JSON.parse(orcamento.Orcamento.seminovo);
                } catch (ex) {
                    orcamento.Orcamento.seminovo = null;
                }
            }

            var usuario_id = orcamento.Orcamento.usuario_id || 0;
            if (usuario_id in UsuariosDB) {
                orcamento['Usuario'] = UsuariosDB[usuario_id];
            } else {
                Usuarios.fetch({
                    query: "SELECT * FROM " + Usuarios.config.adapter.collection_name + " WHERE id = " + usuario_id,
                    success: function (collection) {
                        orcamento['Usuario'] = collection.length ? collection.get(usuario_id).toJSON() : {};

                        UsuariosDB[usuario_id] = orcamento['Usuario'];
                    }
                });
            }

            if (orcamento_id in DetalhesDB) {
                orcamento['OrcamentoDetalhe'] = [DetalhesDB[orcamento_id]];
            } else {
                OrcamentoDetalhes.fetch({
                    query: "SELECT id,created,modified,orcamento_id,data,subtotal,total FROM " + OrcamentoDetalhes.config.adapter.collection_name + " WHERE orcamento_id = '" + orcamento.Orcamento.id + "' ORDER BY data DESC, modified DESC, created DESC LIMIT 0,1",
                    success: function (collection) {
                        orcamento['OrcamentoDetalhe'] = collection.length ? collection.toJSON() : [];
                    }
                });
            }

            return orcamento;
        },
        serializeOrcamentoCompleto: function (orcamento) {
            if (!OrcamentoDetalhes) OrcamentoDetalhes = Alloy.Collections.instance('OrcamentoDetalhe');

            orcamento["OrcamentoDetalhe"] = [];

            OrcamentoDetalhes.fetch({
                query: "SELECT * FROM " + OrcamentoDetalhes.config.adapter.collection_name + " WHERE orcamento_id = '" + orcamento.Orcamento.id + "' ORDER BY data DESC, modified DESC, created DESC",
                success: function (collection) {
                    orcamento['OrcamentoDetalhe'] = collection.length ? collection.toJSON() : [];
                }
            });

            for (var j in orcamento.OrcamentoDetalhe) {
                if (!empty(orcamento.OrcamentoDetalhe[j].opcionais)) {
                    try {
                        orcamento.OrcamentoDetalhe[j].opcionais = JSON.parse(orcamento.OrcamentoDetalhe[j].opcionais);
                    } catch (ex) {
                        orcamento.OrcamentoDetalhe[j].opcionais = null;
                    }
                }
                if (!empty(orcamento.OrcamentoDetalhe[j].entrada)) {
                    try {
                        orcamento.OrcamentoDetalhe[j].entrada = JSON.parse(orcamento.OrcamentoDetalhe[j].entrada);
                    } catch (ex) {
                        orcamento.OrcamentoDetalhe[j].entrada = null;
                    }
                }
                if (!empty(orcamento.OrcamentoDetalhe[j].financiamento)) {
                    try {
                        orcamento.OrcamentoDetalhe[j].financiamento = JSON.parse(orcamento.OrcamentoDetalhe[j].financiamento);
                    } catch (ex) {
                        orcamento.OrcamentoDetalhe[j].financiamento = null;
                    }
                }
            }

            return orcamento;
        },
        syncOrcamento: function (progress) {
            var self = this;
            var deferred = Q.defer();
            var online = Alloy.Globals.isOnline();

            if (!Orcamentos) Orcamentos = Alloy.Collections.instance('Orcamento');
            if (!OrcamentoDetalhes) OrcamentoDetalhes = Alloy.Collections.instance('OrcamentoDetalhe');

            var Usuario = Alloy.Globals.getAuth(),
                params = [],
                where = ["status != 'B'"];
            params.push("usuario_id=" + Usuario.id);

            if (Usuario.type != "A") {
                where.push("usuario_id IN(" + self.usuarioEquipe(Usuario.id).join(",") + ")");
            }

            var currentCache = cache.getItem(Alloy.CFG.defaultUrl + ("/orcamentos/api/list.json%usuario_id=" + Usuario.id + "%"));
            if (currentCache && currentCache.cache && currentCache.created != null) {
                params.push("modified=" + currentCache.created);
            }

            function parseFetch() {
                var start = new Date();

                var ids = [];
                Orcamentos.map(function (orcamentoModel) {
                    ids.push("'" + orcamentoModel.get('id') + "'");
                });

                ids = _.uniq(ids);

                self.fetchOrcamentoDetalhe(ids);

                var results = [];
                Orcamentos.map(function (orcamentoModel) {
                    results.push(self.serializeOrcamento(orcamentoModel));
                });

                var end = new Date();

                Ti.API.log("Tempo: " + (end - start));

                return deferred.resolve(results);
            }

            Orcamentos && Orcamentos.fetch({
                query: "SELECT * FROM " + Orcamentos.config.adapter.collection_name + " WHERE 1 = 1 " + (where.length ? "AND " + where.join(" AND ") : "") + " ORDER BY modified DESC, created ASC",
                success: function (collection) {
                    if (!online && collection.length > 0) {
                        Ti.API.log("Cache offline: orcamentos");
                        return parseFetch();
                    }

                    self.fetchDataIfNeeded({
                        url_fetch: Alloy.Globals.getApiUri("/orcamentos/api/list.json?" + params.join("&"))
                    }).then(function (response) {
                        if (response.isFresh && response.responseData) {
                            self.saveAll(response.responseData, function translateLine(row) {
                                if (!empty(row.Orcamento.seminovo)) {
                                    try {
                                        row.Orcamento.seminovo = JSON.stringify(row.Orcamento.seminovo);
                                    } catch (ex) {
                                        row.Orcamento.seminovo = null;
                                    }
                                }

                                for (var j in row.OrcamentoDetalhe) {
                                    if (!row.OrcamentoDetalhe[j].syncd) row.OrcamentoDetalhe[j].syncd = 1;

                                    if (!empty(row.OrcamentoDetalhe[j].opcionais)) {
                                        try {
                                            row.OrcamentoDetalhe[j].opcionais = JSON.stringify(row.OrcamentoDetalhe[j].opcionais);
                                        } catch (ex) {
                                            row.OrcamentoDetalhe[j].opcionais = null;
                                        }
                                    }
                                    if (!empty(row.OrcamentoDetalhe[j].entrada)) {
                                        try {
                                            row.OrcamentoDetalhe[j].entrada = JSON.stringify(row.OrcamentoDetalhe[j].entrada);
                                        } catch (ex) {
                                            row.OrcamentoDetalhe[j].entrada = null;
                                        }
                                    }
                                    if (!empty(row.OrcamentoDetalhe[j].financiamento)) {
                                        try {
                                            row.OrcamentoDetalhe[j].financiamento = JSON.stringify(row.OrcamentoDetalhe[j].financiamento);
                                        } catch (ex) {
                                            row.OrcamentoDetalhe[j].financiamento = null;
                                        }
                                    }
                                }
                                if (row.Usuario) {
                                    row.Usuario = null;
                                    delete row.Usuario;
                                }
                                return row;
                            }, false, progress);

                            Orcamentos.fetch({
                                query: "SELECT * FROM " + Orcamentos.config.adapter.collection_name + " WHERE 1 = 1 " + (where.length ? "AND " + where.join(" AND ") : "") + " ORDER BY modified DESC, created ASC"
                            });
                            return parseFetch();
                        } else {
                            Ti.API.log("Cache checked online: orcamentos");
                            return parseFetch();
                        }
                    }, function (error) {
                        return deferred.reject([]);
                    });
                }
            });

            return deferred.promise;
        }
    };
}();

module.exports = DataSync;