var syncUsuario = require('syncUsuario');

_.str = require('underscore.string');
_.mixin(_.str.exports());

var LoginController = {
    successCallback: function(){},
    errorCallback: function(){},
    opened: false,
    inSync: false,
    init: function()
    {
        LoginController.toggleButtons(true);

        $.loginFieldLogin.clearButtonMode = Titanium.UI.INPUT_BUTTONMODE_ONFOCUS;
        $.loginFieldLogin.autocapitalization = Titanium.UI.TEXT_AUTOCAPITALIZATION_NONE;
        $.loginFieldLogin.autocorrect = false;

        $.loginFieldPassword.clearButtonMode = Titanium.UI.INPUT_BUTTONMODE_ONFOCUS;
        $.loginFieldPassword.autocapitalization = Titanium.UI.TEXT_AUTOCAPITALIZATION_NONE;
        $.loginFieldPassword.returnKeyType = Titanium.UI.RETURNKEY_GO;
        $.loginFieldPassword.autocorrect = false;
        this.addEventsListener();        

        return this;
    },
    addEventsListener: function()
    {
        $.login.addEventListener("open", _.bind(this.loginOpen, this));
        $.login.addEventListener("close", _.bind(this.loginClose, this));

        $.loginFieldPassword.addEventListener("return", _.bind(this.doLogin, this));
    },
    loginOpen: function(event)
    {
        LoginController.opened = true;

        $.loginFieldLogin.value = Ti.App.Properties.getString("rememberLogin") || "";

        LoginController.syncUsuarios(function(usuarios) {
            LoginController.toggleButtons(LoginController.inSync);
            if(usuarios.length < 1)
            {
                Ti.UI.createAlertDialog({
                    message: L('login_error'),
                    title: L('login_error_title')
                }).show();

                LoginController.disableForm();

                return false;
            }
        });

        LoginController.toggleButtons(LoginController.inSync);
    },
    loginClose: function(event)
    {
        LoginController.opened = false;

        $.destroy();
        $.off();
    },
    registerCallback: function(s, e)
    {
        if(s) LoginController.successCallback = s;
        if(e) LoginController.errorCallback = e;
    },
    syncUsuarios: function(cb)
    {
        if(!LoginController.inSync)
        {
            LoginController.inSync = true;

            syncUsuario.sync(function(data) {
                LoginController.inSync = false;
                if(cb) cb(data);
            });
        }
    },
    enableForm: function()
    {
        $.loginForm.opacity = 1;
        $.loginForm.touchEnabled = true;

        $.loginButton.touchEnabled = true;
        $.loginFieldLogin.editable = true;
        $.loginFieldPassword.editable = true;
    },
    disableForm: function()
    {
        $.loginForm.opacity = 0.5;
        $.loginForm.touchEnabled = false;

        $.loginButton.touchEnabled = false;
        $.loginFieldLogin.editable = false;
        $.loginFieldPassword.editable = false;
    },
    toggleButtons: function(inSync)
    {
        if(!inSync)
        {
            LoginController.enableForm();

            $.syncLabel.hide();
        }
        else
        {
            LoginController.disableForm();

            $.syncLabel.show();
        }
    },
    doLogin: function()
    {
        if($.loginButton.touchEnabled)
        {
            var dataForm = {
                "data[Usuario][username]": _.trim($.loginFieldLogin.value),
                "data[Usuario][password]": _.trim($.loginFieldPassword.value)
            };

            $.loginFieldLogin.blur();
            $.loginFieldPassword.blur();

            Ti.App.Properties.setString("rememberLogin", dataForm["data[Usuario][username]"]);

            if(!Ti.Network.online)
            {
                Ti.UI.createAlertDialog({
                    message: L('login_error_offline'),
                    title: 'Todobit'
                }).show();

                return false;
            }

            if(dataForm["data[Usuario][username]"] != "" && dataForm["data[Usuario][password]"] != "")
            {
                LoginController.disableForm();

                $.syncLabel.message = L('login_sending');
                $.syncLabel.show();

                function successLogin(data)
                {
                    if(data && data.length)
                    {
                        try {
                            var responseData = JSON.parse(data);
                            if("success" in responseData && !!responseData.success && !responseData.error)
                            {
                                var Usuario = responseData.auth;
                                Usuario.ssk = Titanium.Utils.md5HexDigest([Usuario.id, Usuario.username, Usuario.email].join(":"));

                                Ti.API.log("Usuário Logado: " + JSON.stringify(Usuario));

                                Ti.App.Properties.setObject("Usuario", Usuario);

                                Alloy.Globals.refreshLogin();

                                LoginController.successCallback(Usuario);

                                return LoginController.close();
                            }
                        } catch(ex) {
                            Ti.API.error("Erro ao processar login" + ex.toString());
                            return errorLogin(data);
                        }
                    }

                    return errorLogin(data);
                };

                function errorLogin(data)
                {
                    Alloy.Globals.refreshLogin();

                    Ti.UI.createAlertDialog({
                        message: L('login_error_data'),
                        title: 'Todobit'
                    }).show();

                    LoginController.enableForm();
                    $.syncLabel.hide();

                    LoginController.errorCallback(data);
                };

                LoginController.requestLogin({
                    success: successLogin,
                    error: errorLogin
                }, dataForm);

            }
        }
    },
    requestLogin: function(_options, formData)
    {
        var options = _.extend({
            url: Alloy.CFG.defaultUrl + "/usuarios/login",
            timeout: (60 * 1000),
            async: true,
            cache: false,
            success: function(data){},
            error: function(data){}
        }, _options);

        var xhr = Ti.Network.createHTTPClient(options);
        xhr.onload = function(event)
        {
            options.success(this.responseText);
        };
        xhr.onerror = function(event)
        {
            options.error(xhr.responseText, event.error);
        };
        xhr.open("POST", options.url);
        xhr.send(formData||{});
    },
    
    close: function()
    {
        $.login.close();
    }
};

exports = LoginController.init();
