
if (typeof Alloy === 'undefined') {
    var Alloy = require('alloy');
}
if (typeof Backbone === 'undefined') {
    var Backbone = require('alloy/backbone');
}
if (typeof _ === 'undefined') {
    var _ = require('alloy/underscore')._;
}

var cache = require('cache');

var Net = {
    fetchImage: function(_options)
    {
        var imageQueue = Alloy.Globals.imageQueue;

        var _options = _.extend({
            defaultUrl: Alloy.CFG.defaultUrl,
            thumbUrl: Alloy.CFG.thumbUrl,
            image: false,
            headers: [],
            cache: true,
            md5sum: 'cd7a185a84a333cf94f030e2fbb86877',
            success: function(data) {},
            error: function(data) {},
            progress: function(data) {},
        }, (_options||{}));

        if(!_options.image)
        {
            Ti.API.error('Please inform an IMAGE to request');
            return false;
        }

        var openCacheDir = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, Alloy.CFG.imageCacheDir);
        if(!openCacheDir.exists()) openCacheDir.createDirectory();

        (function(path) {
            path = path.split("/");
            realpath = Alloy.CFG.imageCacheDir;
            for(var j=0; j<path.length-1; j++)
            {
                realpath = realpath + "/" + path[j];

                var tempDir = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, realpath);
                if(!tempDir.exists()) tempDir.createDirectory();

                try { tempDir.setRemoteBackup(false); } catch(ex) {}
            }
        })(_options.image);

        var imageCache = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, Alloy.CFG.imageCacheDir + "/" + _options.image);
        var imageCacheSum = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, Alloy.CFG.imageCacheDir + "/" + _options.image + ".md5");

        /* COPY ORIGINAL */
        if(_options.cache && !imageCache.exists())
        {
            var baseFile = Ti.Filesystem.getFile(Ti.Filesystem.resourcesDirectory + Alloy.CFG.imageCacheDir + "/" + _options.image);
            var baseFileSum = Ti.Filesystem.getFile(Ti.Filesystem.resourcesDirectory + Alloy.CFG.imageCacheDir + "/" + _options.image + ".md5");
            if(baseFile.exists() && baseFileSum.exists())
            {
            	baseFile.setRemoteBackup(false);

                var image_md5sum = false;
                try {
                    if(baseFileSum.exists())
                    {
                    	baseFileSum.setRemoteBackup(false);

                        var blob = baseFileSum.read();
                        image_md5sum = blob.text;
                    }
                } catch(ex) {}

                if(_options.md5sum === image_md5sum)
                {
                    Ti.API.log("Feed by original: " + _options.image);

                    try { imageCache.write(baseFile.read(), false); imageCache.setRemoteBackup(false); } catch(ex) { }
                    try { imageCacheSum.write(baseFileSum.read(), false); imageCacheSum.setRemoteBackup(false); } catch(ex) { }
                }
            }
        }

        if(_options.cache && imageCache.exists())
        {
        	try { imageCache.setRemoteBackup(false); } catch(ex) { }

            var image_md5sum = false;
            try {
                if(imageCacheSum.exists())
                {
                	imageCacheSum.setRemoteBackup(false);

                    var blob = imageCacheSum.read();
                    image_md5sum = blob.text;
                }
            } catch(ex) {}

            if(_options.md5sum === image_md5sum)
            {
                Ti.API.log("Feed by cache: " + _options.image);

                var successData = {
                    cache: true,
                    imageData: imageCache.read(),
                    imagePath: imageCache.nativePath
                };

                if(imageQueue[_options.image])
                {
                    for(var h=0; h<imageQueue[_options.image].length; h++)
                    {
                        if(typeof imageQueue[_options.image][h].success == "function")
                        {
                            imageQueue[_options.image][h].success(successData);
                        }
                    }

                    imageQueue[_options.image] = null;
                    delete imageQueue[_options.image];
                }

                _options.success(successData);
                return;
            }
            else
            {
                if(imageCache.exists()) imageCache.deleteFile();
                if(imageCacheSum.exists()) imageCacheSum.deleteFile();
            }
        }

        if(Alloy.Globals.isOnline() || !_options.cache)
        {
            var imageUrl = _options.image.indexOf("http") === 0 ? _options.image : (_options.thumbUrl + _options.image);

            if(!imageQueue[_options.image])
            {
                Ti.API.log("Fetching image: " + _options.image);

                var xhrImage = Ti.Network.createHTTPClient({
                    cache: false
                });
                xhrImage.onload = function(e) {
                    imageCache.write(this.responseData, false);
                    imageCache.setRemoteBackup(false);

                    imageCacheSum.write(_options.md5sum, false);
                    imageCacheSum.setRemoteBackup(false);

                    var successData = {
                        cache: false,
                        imageData: imageCache.read(),
                        imagePath: imageCache.nativePath
                    };

                    if(imageQueue[_options.image])
                    {
                        for(var h=0; h<imageQueue[_options.image].length; h++)
                        {
                            if(typeof imageQueue[_options.image][h].success == "function")
                            {
                                imageQueue[_options.image][h].success(successData);
                            }
                        }

                        imageQueue[_options.image] = null;
                        delete imageQueue[_options.image];
                    }
                };
                xhrImage.onerror = function(e) {
                    if(imageCache.exists()) imageCache.deleteFile();
                    if(imageCacheSum.exists()) imageCacheSum.deleteFile();

                    if(imageQueue[_options.image])
                    {
                        for(var h=0; h<imageQueue[_options.image].length; h++)
                        {
                            if(typeof imageQueue[_options.image][h].error == "function")
                            {
                                imageQueue[_options.image][h].error(e);
                            }
                        }

                        imageQueue[_options.image] = null;
                        delete imageQueue[_options.image];
                    }
                };
                xhrImage.ondatastream = function(e) {
                    if(imageQueue[_options.image])
                    {
                        for(var h=0; h<imageQueue[_options.image].length; h++)
                        {
                            if(typeof imageQueue[_options.image][h].progress == "function")
                            {
                                imageQueue[_options.image][h].progress(e);
                            }
                        }
                    }
                };
                xhrImage.open('GET', imageUrl);
                xhrImage.send();

                imageQueue[_options.image] = [];
            }

            imageQueue[_options.image].push(_options);
        }

        Alloy.Globals.imageQueue = imageQueue;
    },
    fetchData: function(_options, _callback)
    {
        var _options = _.extend({
            type: "GET",
            headers: {
                'Accept': 'text/html,application/json,application/xml,application/xhtml+xml',
                'Accept-Encoding': 'gzip,deflate',
                'X-App': Ti.App.id,
                'X-App-Version': Ti.App.version
            },
            cache: true,
            success: function(data){},
            progress: function(event, state){},
            error: false
        }, (_options||{}));

        if(_callback)
        {
            _options.success = _callback;
        }

        if(!_options.error && _options.success)
        {
            _options.error = _options.success;
        }

        if(!_options.url)
        {
            Ti.API.error('Please inform an URL to request');
            return false;
        }

        var cacheData = "md5sum" in _options ? cache.getItem(_options.url, _options.md5sum) : cache.getItem(_options.url);

        if(Alloy.Globals.isOnline() || !_options.cache)
        {
            var xhr = Ti.Network.createHTTPClient({
                timeout: (20 * 1000),
                async: _options.async || true,
                active: false,
                cache: !_options.cache
            });

            xhr.close = function()
            {
                if(xhr.active)
                {
                    xhr.active = false;
                    xhr.abort();
                }
            };

            xhr.onreadystatechange = _options.onreadystatechange || function(event) {
                // Ti.API.log("R: " + this.readyState);
            };

            xhr.onsendstream = function(event)
            {
                _options.progress(event.progress, xhr.readyState);
            };

            // Handle load
            xhr.onload = function(e) {
                var md5sum = null;
                var responseJSON = null;
                var contentType = xhr.getResponseHeader("Content-type") || "";
                if(contentType.indexOf("text/plain") > -1 || contentType.indexOf("json") > -1)
                {
                    try {
                        responseJSON = JSON.parse(this.responseData);
                        if("md5sum" in responseJSON) md5sum = responseJSON['md5sum'];
                    } catch(ex) {}
                }

                if(_options.cache)
                {
                    cache.setItem(_options.url, this.responseData, md5sum);
                }

                _options.success({
                    success: true,
                    cache: false,
                    contentType: contentType,
                    responseJSON: responseJSON || null,
                    responseText: this.responseText || null,
                    responseData: this.responseData || null
                });
            };

            // Handle error
            xhr.onerror = _.bind(function(e) {
                Ti.API.error('[REST API] fetchData ERROR: ' + xhr.responseText);

                if(!_options.cache || !cacheData.cache)
                {
                    Ti.API.error('No cache data available to use');

                    _options.error({
                        'cache': false,
                        'success': false,
                        'responseText': e.error,
                        'error': e.error
                    });
                }
                else
                {
                    Ti.API.warn('Using cache data instead');

                    var responseJSON = null;
                    try {
                        responseJSON = JSON.parse(cacheData.data);
                    } catch(ex) {}

                    _options.success({
                        success: true,
                        cache: true,
                        responseJSON: responseJSON || null,
                        responseText: cacheData.data || null,
                        responseData: cacheData.data || null
                    });
                }
            }, this);

            Ti.API.log('[REST API] fetchData: ' + _options.url);

            //Prepare the request
            xhr.open(_options.type || "GET", _options.url);

            for (var header in _options.headers) {
                xhr.setRequestHeader(header, _options.headers[header]);
            }

            if (_options.beforeSend) {
                _options.beforeSend(xhr);
            }

            xhr.send(_options.data || null);
            xhr.active = true;

            return xhr;
        }
        else
        {
            if(cacheData && cacheData.cache)
            {
                var responseText = null;
                var responseJSON = null;
                try {
                    responseJSON = JSON.parse(cacheData.data);
                    responseText = cacheData.data.toString();
                } catch(ex) {}

                if(responseJSON == null && responseText == null)
                {
                    try { responseText = cacheData.data.toString(); } catch(ex) {}
                }

                _options.success({
                    success: true,
                    cache: true,
                    contentType: null,
                    responseJSON: responseJSON,
                    responseText: responseText,
                    responseData: cacheData.data || null
                });
            }
            else
            {
                _options.error({
                    'cache': false,
                    'success': false,
                    'responseText': 'no internet connection and no cache file found',
                    'error': 'no internet connection and no cache file found'
                });

                Ti.API.error('[REST API] fetchData ERROR: no internet connection and no cache file found');
                return false;
            }

            return null;
        }
    }
};

module.exports = Net;
