if (typeof Alloy === 'undefined') {
    var Alloy = require('alloy');
}
if (typeof Backbone === 'undefined') {
    var Backbone = require('alloy/backbone');
}
if (typeof _ === 'undefined') {
    var _ = require('alloy/underscore')._;
}

var Q = require('q');
var net = require('net');
var cache = require('cache');

function empty(v)
{
    return (!v || (typeof v == "string" && v == "") || v == null);
}

var SU = (function() {

    var Usuarios = null;
    var UsuarioGerentes = null;

    return {
        sync: function(success, error, progress)
        {
            var success = (success || function(){}),
                error = (error || function(){}),
                progress = (progress || function(){});

            return this.syncUsuario(progress).then(success, error);
        },
        fetchDataIfNeeded: function(_options)
        {
            var self = this;
            var deferred = Q.defer();
            var online = Alloy.Globals.isOnline();

            var md5sum = null, modified = null;

            var _options = _.extend({}, (_options||{}));

            var currentFetch = cache.getItem(_options.url_fetch);
            if(currentFetch.cache && currentFetch.md5sum != null && currentFetch.md5sum != "")
            {
                md5sum = currentFetch.md5sum;
                modified = currentFetch.created;
            }

            net.fetchData({
                url: _options.url_fetch,
                success: function(data) {
                    if(data.success && "responseJSON" in data)
                    {
                        try {
                            if("md5sum" in data.responseJSON)
                            {
                                return deferred.resolve({
                                    responseData: data.responseJSON.data,
                                    actual_md5sum: data.responseJSON.md5sum,
                                    isFresh: (data.responseJSON.md5sum != md5sum)
                                });
                            }
                        } catch(ex) {
                        }
                    }

                    return deferred.resolve({
                        responseData: false,
                        actual_md5sum: md5sum,
                        isFresh: false
                    });
                },
                error: function(data)
                {
                    return deferred.reject({
                        responseData: false,
                        actual_md5sum: md5sum,
                        isFresh: false
                    });
                }
            });

            return deferred.promise;
        },
        save: function(model, data)
        {
            var newEntry = Alloy.createModel(model, data);
            newEntry.save();

            return newEntry.id;
        },
        saveAll: function(rows, translateLine, deleteOthers, progress)
        {
            deleteOthers = (typeof deleteOthers == "undefined" ? true : !!deleteOthers);

            var self = this;
            var rows = (rows || []);
            var progress = (progress || function(){});
            var translateLine = (translateLine || function(row) {
                return row;
            });

            var updatedIds = {};

            var db = Alloy.Globals.openDatabase();
            function internalSave(model, data)
            {
                var newEntry = Alloy.createModel(model, data);

                var table = newEntry.config.adapter.collection_name,
                    sql, columns = newEntry.config.columns;

                var names = [], values = [], q = [], u = [], qv = [];
                for (var k in columns) {
                    if(k in data) {
                        names.push(k);
                        values.push(data[k]);
                        q.push("?");
                        qv.push(!isNaN(data[k]) ? data[k] : "'" + data[k] + "'");
                        u.push(k + " = ?");
                    }
                }

                // Ti.API.log("REPLACE INTO " + table + " (" + names.join(",") + ") VALUES (" + qv.join(",") + ");");
                sql = "REPLACE INTO " + table + " (" + names.join(",") + ") VALUES (" + q.join(",") + ");";
                db.execute(sql, values);
                if (!("id" in data) || null === data['id']) {
                    var sqlId = "SELECT last_insert_rowid();";
                    var rs = db.execute(sqlId);
                    if (rs && rs.isValidRow()) {
                        data['id'] = rs.field(0);
                    } else Ti.API.warn("Unable to get ID from database for model: " + model);
                    rs && rs.close();
                }

                return ("id" in data) ? data['id'] : false;
            };

            Ti.API.log("[DATA SYNC] SAVING NEW ROWS");

			db.execute("BEGIN IMMEDIATE TRANSACTION");

            /* SAVE NEW ROWS */
           	var db_rows = (_.isArray(rows) ? rows : [rows]);
            _.each(db_rows, function(row, index) {
                var newRow = translateLine(row);
                if(newRow)
                {
                    _.each(newRow, function(data, model) {
                        if(!updatedIds[model]) updatedIds[model] = [];

                        if(_.isArray(data))
                        {
                            _.each(data, function(subdata, subindex) {
                                var newEntry = internalSave(model, subdata);
                                if(newEntry) updatedIds[model].push(!isNaN(subdata.id) ? subdata.id : "'" + subdata.id + "'");
                            });
                        }
                        else
                        {
                            var newEntry = internalSave(model, data);
                            if(newEntry) updatedIds[model].push(!isNaN(data.id) ? data.id : "'" + data.id + "'");
                        }
                    });
                }
                progress((index+1), db_rows.length);
            });

            /* DELETE POSSIBLE OLD ROWS */
            if(deleteOthers)
            {
                Ti.API.warn("[DATA SYNC] DELETE POSSIBLE OLD ROWS");

                _.each(updatedIds, function(ids, model) {
                    var theModel = Alloy.Collections.instance(model);
                    if(theModel)
                    {
                        var table = theModel.config.adapter.collection_name;
                        var delete_sql = "DELETE FROM " + table;
                        if(ids.length)
                        {
                            delete_sql += " WHERE id NOT IN(" + ids.join(", ") + ")";
                        }

                        db.execute(delete_sql);
                    }
                });
            }

			db.execute("COMMIT TRANSACTION");
            db.close();

            return updatedIds;
        },
        serializeUsuario: function(usuarioModel)
        {
            if(!Usuarios) Usuarios = Alloy.Collections.instance('Usuario');
            if(!UsuarioGerentes) UsuarioGerentes = Alloy.Collections.instance('UsuarioGerente');

            var usuario = {};
            usuario["Usuario"] = usuarioModel.toJSON();

            return usuario;
        },
        syncUsuario: function(progress)
        {
            var self = this;
            var deferred = Q.defer();
            var online = Alloy.Globals.isOnline();

            if(!Usuarios) Usuarios = Alloy.Collections.instance('Usuario');
            if(!UsuarioGerentes) UsuarioGerentes = Alloy.Collections.instance('UsuarioGerente');

            function parseFetch()
            {
                var results = [];
                Usuarios.map(function(usuarioModel) {
                    results.push(self.serializeUsuario(usuarioModel));
                });

                return deferred.resolve(results);
            }

            Usuarios && Usuarios.fetch({
                success: function(collection) {
                    if(!online && collection.length > 0)
                    {
                        Ti.API.log("Cache offline: usuarios");
                        return parseFetch();
                    }

                    self.fetchDataIfNeeded({
                        url_fetch: Alloy.Globals.getApiUri("/usuarios/api/list.json"),
                    }).then(function(response) {
                        if(response.isFresh && response.responseData)
                        {
                            self.saveAll(response.responseData, function translateLine(row) {
                                row['Usuario']['password'] = "";
                                return row;
                            }, true, progress);

                            Usuarios.fetch();
                            return parseFetch();
                        }
                        else
                        {
                            Ti.API.log("Cache checked online: usuarios");
                            return parseFetch();
                        }
                    });
                }
            });

            return deferred.promise;
        },
        usuarioEquipe: function(usuario_id)
        {
            return Alloy.Globals.getUsuarioAndEquipe(usuario_id);
        }
    };

})();

module.exports = SU;
